﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

namespace Sample1
{
    public partial class Form1 : Form
    {
        // One and only Device Index:
        public UInt32 DeviceIndex = AIOUSB.diOnly;
        public readonly UInt32 numPins = 32;
        public readonly UInt32 numPorts = 4;
        public readonly UInt16 number_to_histogram = 5000;

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        uint[] histogram_data = new uint[65536];
        uint number_of_readings = 0;
        uint MinReading, MaxReading, ModeReading, MeanReading;


        public Form1()
        {
            InitializeComponent();
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:
            DeviceIndex = AIOUSB.GetDeviceByEEPROMByte(1);
            myTimer.Tick += new EventHandler(TimerEventProcessor);
            myTimer.Stop();
            // Set the timer interval in miliseconds and start 55ms min resolution:
            myTimer.Interval = 1; // as fast as possible: timer turns itself off during processing to avoid going too fast

            UInt32 Outs = 0x04;
            UInt32 Data = 0xFFAFFFFF;

            AIOUSB.DIO_Configure(DeviceIndex, 0, ref Outs, ref Data);
        }

        private string calculateHistogram(uint ADdata)
        {
            string HistogramTabular = "";

            histogram_data[ADdata]++;
            number_of_readings++;

            if (ADdata > MaxReading) MaxReading = ADdata;
            if (ADdata < MinReading) MinReading = ADdata;

            if (number_of_readings >= number_to_histogram)
            {
                uint sumReading = 0;

                MeanReading = 0;
                ModeReading = 0;

                for (uint i = 0; i < histogram_data.Count(); i++)
                {
                    sumReading += histogram_data[i]*i;
                    if (histogram_data[i] > histogram_data[ModeReading]) ModeReading = i;
                }
                MeanReading = (UInt16)(sumReading / number_to_histogram);

                for (uint i= MinReading; i <= MaxReading; i++)
                {

                    HistogramTabular += i.ToString("X4");
                    if (histogram_data[i] > 0) HistogramTabular += ":" + histogram_data[i].ToString();
                    HistogramTabular += "\r\n";

                }

                return "Num :" + number_of_readings.ToString() + "\r\n" +
                       "Max : " + MaxReading.ToString("X4") + "  Min : " + MinReading.ToString("X4") + "\r\n" +
                       "Mode: " + ModeReading.ToString("X4") + " Mean: " + MeanReading.ToString("X4") + "\r\n" + 
                       HistogramTabular ;
            }
            else return "";
        }

        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            myTimer.Stop();

            string Histogram_Formatted = calculateHistogram(GetImmediate());

            if (Histogram_Formatted == "")
            {
                progressBar1.Value++;
                myTimer.Start();
            }
            else
            {
                myTimer.Stop();
                textBox1.Clear();
                textBox1.AppendText(Histogram_Formatted);
                number_of_readings = 0;
                Histogram_Formatted = "";
                progressBar1.Value = 0;
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {

        }

        private UInt16 GetImmediate()
        {
            byte[] ADCData = new byte[4];
            
            AIOUSB.DIO_Write1(DeviceIndex, 16, 0); // set CONVST LOW
            AIOUSB.DIO_Write1(DeviceIndex, 16, 1); // set CONVST HIGH

            AIOUSB.DIO_Write1(DeviceIndex, 17, 0); // set READ LOW
            AIOUSB.DIO_ReadAll(DeviceIndex, ADCData);
            AIOUSB.DIO_Write1(DeviceIndex, 17, 1); // set READ HIGH

            return (UInt16)(ADCData[0] + (ADCData[1] << 8));
        }

        private void butREADADC_Click(object sender, EventArgs e)
        {
            UInt16 counts = GetImmediate();

            histogram_data[counts]++;
            listBox1.Items.Add(counts.ToString("d5") + " : " + counts.ToString("X4"));
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            progressBar1.Maximum = number_to_histogram;

            System.Drawing.Graphics graphics = this.CreateGraphics();
            System.Drawing.Rectangle rectangle = new System.Drawing.Rectangle(300, 100, 200, 200);
            SolidBrush blueBrush = new SolidBrush(Color.Blue);

            graphics.DrawEllipse(System.Drawing.Pens.Black, rectangle);
            graphics.FillRectangle(blueBrush, rectangle);

            MaxReading = 0;
            MinReading = 0xFFFF;

            myTimer.Start();
        }
    }
}


// UInt16 adCounts_saywhat = (UInt16)(ADCData[0] + ADCData[1] << 8);  // SHIFT HAS LOWER PRECEDENCE THAN PLUS?!?!?
